import 'dart:convert';
import 'dart:io';

import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:get/get.dart';
import 'package:rxdart/rxdart.dart';

class NotificationService {
  GlobalKey<NavigatorState> get _navigatorKey {
    return Get.key;
  }

  BuildContext get _context {
    return _navigatorKey.currentState.overlay.context;
  }

  // ignore: close_sinks
  BehaviorSubject<String> tokenSubject = BehaviorSubject<String>();
  // ignore: close_sinks
  PublishSubject<Map<String, dynamic>> selectSubject =
      PublishSubject<Map<String, dynamic>>();
  // ignore: close_sinks

  setup() async {
    await _registerListener();
    await _setupLocalNotification();
  }

  FirebaseMessaging _firebaseMessaging = FirebaseMessaging();
  FlutterLocalNotificationsPlugin _localNotification =
      new FlutterLocalNotificationsPlugin();

  _registerListener() async {
    final fcmToken = await _firebaseMessaging.getToken();
    tokenSubject.add(fcmToken);
    print('fcmToken $fcmToken');

    _firebaseMessaging.configure(
      onMessage: (Map<String, dynamic> message) async {
        print('on message $message');
        _displayLocalNotification(message);
      },
      onResume: (Map<String, dynamic> message) async {
        var data = message['data'] ?? message;
        print('on resume $data');
        _getdata(json.encode(data));
      },
      onLaunch: (Map<String, dynamic> message) async {
        var data = message['data'] ?? message;
        print('on launch $data');
        await Future.delayed(Duration(seconds: 5));
        _getdata(json.encode(data));
      },
    );
  }

  _setupLocalNotification() {
    final initAndroidSettings =
        new AndroidInitializationSettings('@mipmap/ic_launcher');
    final initIOSSettings = new IOSInitializationSettings();

    final initSettings = new InitializationSettings(
        android: initAndroidSettings, iOS: initIOSSettings);

    _localNotification.initialize(initSettings,
        onSelectNotification: _onSelectLocalNotification);
  }

  _getdata(String payload) {
    // logger.d('_getdata: $payload');
    // if (onHandleData == null) {
    //   logger.e('onHandleData not set');
    //   return;
    // }
    // onHandleData(payload);
  }

  Future _displayLocalNotification(Map<String, dynamic> message) async {
    var androidDetails = new AndroidNotificationDetails(
        'channelid', 'flutterfcm', 'your channel description',
        importance: Importance.max, priority: Priority.high);

    var iOSDetails = new IOSNotificationDetails();

    var platformChannelSpecifics =
        new NotificationDetails(android: androidDetails, iOS: iOSDetails);

    if (Platform.isAndroid) {
      await _localNotification.show(
        0,
        message['notification']['title'],
        message['notification']['body'],
        platformChannelSpecifics,
        payload: json.encode(message['data']),
      );
    } else {
      var data = message['data'] ?? message;
      final title = data['aps']['alert']['title'];
      final body = data['aps']['alert']['body'];
      await _localNotification.show(
        0,
        title,
        body,
        platformChannelSpecifics,
        payload: json.encode(data),
      );
    }
  }

  Future _onSelectLocalNotification(String payload) async {
    if (payload != null) {
      debugPrint('notification payload: ' + payload);
      _getdata(payload);
    }
  }

  requestPermission() {
    if (Platform.isIOS) {
      _firebaseMessaging.requestNotificationPermissions(
          IosNotificationSettings(sound: true, badge: true, alert: true));
      _firebaseMessaging.onIosSettingsRegistered
          .listen((IosNotificationSettings settings) {
        print("Settings registered: $settings");
      });
    }
  }

  @override
  String get fcmToken => tokenSubject.value ?? '';
}
